<?php
/*
    community.php - A OneSwarm community feed implemented in PHP.
    Copyright (C) 2009, me@maeyanie.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/
require "config.php";
$db = new MySQLi($db_host, $db_user, $db_pass, $db_schema);

function error($code, $msg) {
	error_log($msg);
	header("HTTP/1.0 $code");

	ob_start();
	phpinfo();
	echo "Error: $msg";
	file_put_contents("error.html", ob_get_clean());
	exit;
}
function ltob($num) {
	// Yes, this could probably be done with a single pack(), but I had a heck of a time getting it to work at all.
	$ret = "";
	for ($x = 0; $x < 8; $x++) {
		$ret .= pack("C", ($num >> ((7-$x)*8)) & 0xFF);
	}
	return $ret;
}
function sendpeers() {
	global $peermethod;
	
	switch ($peermethod) {
	default:	
	case 1: // Standard mode -- send "near" keys. Far more expensive than it should be... must find a better way...
		global $peerlimit;
		$key = addslashes($_GET['base64key']);
		$res = $db->query("SELECT base64key,nick FROM os_keys WHERE id != $id AND STRCMP(base64key, $key) == -1 "
			."ORDER BY base64key DESC LIMIT ".($peerlimit/2)) or die($db->error);
		while ($row = $res->fetch_row()) {
			echo "<Friend Base64Key='$row[0]' nick='$row[1]' />";
		}
		$res->close();
		$res = $db->query("SELECT base64key,nick FROM os_keys WHERE id != $id AND STRCMP(base64key, $key) == 1 "
			."ORDER BY base64key ASC LIMIT ".($peerlimit/2)) or die($db->error);
		while ($row = $res->fetch_row()) {
			echo "<Friend Base64Key='$row[0]' nick='$row[1]' />";
		}
		$res->close();
		break;
		
	case 2: // Infrastructure mode -- send certain keys to everyone, except to those keys, who get all keys.
		global $id, $infrapeers;
		if (in_array($id, $infrapeers)) {
			$res = $db->query("SELECT base64key,nick FROM os_keys WHERE id NOT IN (".implode(",", $infrapeers).")") or die($db->error);
		} else {
			$res = $db->query("SELECT base64key,nick FROM os_keys WHERE id IN (".implode(",", $infrapeers).")") or die($db->error);
		}
		while ($row = $res->fetch_row()) {
			echo "<Friend Base64Key='$row[0]' nick='$row[1]' />";
		}
		$res->close();			
		break;

	case 3:	// Small network mode -- send all keys to everyone.
		$res = $db->query("SELECT base64key,nick FROM os_keys WHERE id != $id") or die($db->error);
		while ($row = $res->fetch_row()) {
			echo "<Friend Base64Key='$row[0]' nick='$row[1]' />";
		}
		$res->close();
		break;
	}
}




if ($_SERVER['REQUEST_METHOD'] == "POST") { // Registration request
	if (!isset($_POST["base64key"])) error("400 Bad Request", "No key provided.");
	$key = addslashes($_POST['base64key']);
	
	if (!isset($_POST["nick"])) error("400 Bad Request", "No nick provided.");
	$nick = addslashes($_POST['nick']);

	$res = $db->query("INSERT INTO os_keys (base64key, nick, lastip) VALUES ('$key', '$nick', INET_ATON('".$_SERVER["REMOTE_ADDR"]."'))");
	if (!$res) {
		echo "REGISTRATION_DUPLICATE";
	} else {
		echo "REGISTRATION_OK";
	}
	
} else if ($_SERVER['REQUEST_METHOD'] == "GET") { // Peer fetch request
	if (!isset($_GET["base64key"])) error("400 Bad Request", "No key provided.");
	$key = addslashes($_GET['base64key']);
	
	$res = $db->query("SELECT id FROM os_keys WHERE base64key='$key'") or die($db->error);
	$row = $res->fetch_row();
	if (!$row) {
		header("HTTP/1.0 401 Unauthorized");
		exit;
	}
	$id = $row[0];
	$res->close();
	
	$db->query("UPDATE os_keys SET lastip=INET_ATON('".$_SERVER["REMOTE_ADDR"]."') WHERE id=$id");
	
	if (mt_rand(1, $reapchance) == 1) $db->query("DELETE FROM os_keys WHERE tstamp < NOW() - INTERVAL $reaptime SECOND") or die($db->error);
	
	if (!isset($_GET['resp'])) { // Issue challenge
		$chal = mt_rand();
		$db->query("REPLACE INTO os_challenges (id, challenge) VALUES ($id, $chal)") or die($db->error);
		echo "CHALLENGE $chal";
	} else { // Process response
		$resp = $_GET['resp'];
		$res = $db->query("SELECT challenge FROM os_challenges WHERE id=$id") or die($db->error);
		$row = $res->fetch_row();
		if (!$row) error("401 Unauthorized", "No recent challenge for key.");
		$chal = $row[0];
		$res->close();
		$db->query("DELETE FROM os_challenges WHERE id=$id");
		
		
		$key = openssl_pkey_get_public("-----BEGIN PUBLIC KEY-----\n".chunk_split($key, 64)."-----END PUBLIC KEY-----\n");
		if ($key === false) error("400 Bad Request", "Error decoding key.");
		
		$resp = base64_decode($resp, true);
		if ($resp === false) error("400 Bad Request", "Error decoding response.");
		
		$ok = openssl_verify(ltob($chal+1), $resp, $key);
		if ($ok == 1) { // Good.
		} else if ($ok == 0) { // Bad.
			error("403 Forbidden", "Signature did not match.");
		} else { // Ugly.
			error("400 Bad Request", "Error checking signature.");
		}
		
		
		// Everything checks out! Send some peers.
		echo "<?xml version='1.0'?>";
		echo "<CommunityServerResponse>";
		if (isset($refresh)) echo "<RefreshInterval>$refresh</RefreshInterval>";
		echo "<FriendList>";
		sendpeers();
		echo "</FriendList>";
		echo "</CommunityServerResponse>";
	}
} else {
	error("500 Server Error", "Unhandled method: ".$_SERVER['REQUEST_METHOD']);
}






?>
